-- Globals
CT_RABoard = { };
CT_RABoard.objects = { };
CT_RABoard.color = { ["r"] = 0, ["g"] = 0, ["b"] = 0, ["a"] = 1};
CT_RABoard.lineWidth = 3;

-- Locals
local NUM_OBJECTS = 30;
local framePositions = { };
local DRAWING_OBJECT;
local MOUSE_OBJECT;
local LAST_X, LAST_Y;
local distA, distB, distC;

function CT_RABoard_GetCursorPosition()
	local cx, cy = GetCursorPosition();
	local w, h, wr, hr;
	if ( framePositions[1] ) then
		w, h, wr, hr = unpack(framePositions);
	else
		local obj = CT_RABoardFrameDrawFrame;
		local fx, fy = obj:GetLeft(), obj:GetBottom();
		local fw, fh = obj:GetWidth(), obj:GetHeight();
		
		obj = UIParent;
		wr, hr = obj:GetWidth()/obj:GetRight(), obj:GetHeight()/obj:GetTop();
		w = wr*fx;
		h = hr*fy;
		framePositions = { w, h, wr, hr, fw, fh };
	end
	return ( cx - w ) / wr, ( cy - h ) / hr;
end

function CT_RABoard_AddObject(index, startX, startY, endX, endY, width, r, g, b, a, isDrawing)
	local objects = CT_RABoard.objects;
	local num_objects = getn(objects);
	if ( num_objects < NUM_OBJECTS ) then
		-- Only do the math if we have to...
		local fw, fh;
		if ( framePositions[1] ) then
			fw = framePositions[5];
			fh = framePositions[6];
		else
			-- If the table is not filled already, we might as well fill it instead of only calculatting fw/fh (since then we do that twice when we fill it later)
			local fx, fy = CT_RABoardFrameDrawFrame:GetLeft(), CT_RABoardFrameDrawFrame:GetBottom();
			local fw, fh = CT_RABoardFrameDrawFrame:GetWidth(), CT_RABoardFrameDrawFrame:GetHeight();
			local wr, hr = UIParent:GetWidth()/UIParent:GetRight(), UIParent:GetHeight()/UIParent:GetTop();
			framePositions = { wr*fx, hr*fy, wr, hr, fw, fh };
		end
		if ( startX < 0 ) then
			startX = 0;
		elseif ( startX > fw ) then
			startX = fw;
		end
		if ( startY < 0 ) then
			startY = 0;
		elseif ( startY > fh ) then
			startY = fh;
		end
		if ( endX and endY ) then
			if ( endX < 0 ) then
				endX = 0;
			elseif ( endX > fw ) then
				endX = fw;
			end
			if ( endY < 0 ) then
				endY = 0;
			elseif ( endY > fh ) then
				endY = fh;
			end
		end
		
		num_objects = num_objects + 1;
		table.setn(objects, num_objects);
		objects[index] = { ["startX"] = startX, ["startY"] = startY, ["endX"]= endX, ["endY"] = endY, ["width"] = width, ["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a, ["objID"] = num_objects };
		
		-- Show the object
		local obj = getglobal("CT_RABoardFrameDrawFrameLine" .. num_objects);
		obj:Show();
		if ( isDrawing ) then
			DRAWING_OBJECT = index;
		end
	end
end

function CT_RABoard_ClearBoard(clearID)
	local objects = CT_RABoard.objects;
	if ( clearID ) then
		table.setn(objects, getn(objects)-1);
		getglobal("CT_RABoardFrameDrawFrameLine" .. objects[clearID].objID):Hide();
		getglobal("CT_RABoardFrameDrawFrameLine" .. objects[clearID].objID .. "Texture"):SetTexCoord(0,0,0,0,0,0,0,0);
		objects[clearID] = nil;
	else
		table.setn(objects, 0);
		CT_RABoard.objects = { };
		for i = 1, NUM_OBJECTS, 1 do
			getglobal("CT_RABoardFrameDrawFrameLine" .. i):Hide();
			getglobal("CT_RABoardFrameDrawFrameLine" .. i .. "Texture"):SetTexCoord(0,0,0,0,0,0,0,0);
		end
	end
end

function CT_RABoard_StartDrawing(button)
	if ( button == "LeftButton" ) then
		local cx, cy = CT_RABoard_GetCursorPosition();
		local color = CT_RABoard.color;
		CT_RABoard_AddObject(GetTime(), cx, cy, nil, nil, CT_RABoard.lineWidth, color.r, color.g, color.b, color.a, true);
	end
end

function CT_RABoard_EndDrawing(button)
	if ( DRAWING_OBJECT and button == "LeftButton" ) then
		local cx, cy = CT_RABoard_GetCursorPosition();
		local obj = CT_RABoard.objects[DRAWING_OBJECT];
		obj.endX = cx;
		obj.endY = cy;
		CT_RABoard_DrawMovingObject(DRAWING_OBJECT, cx, cy);
		DRAWING_OBJECT = nil;
		LAST_ANGLE = nil;
		-- If we are in "broadcast mode", broadcast it
		if ( CT_RABoard.isBroadcasting ) then
			CT_RABoard_BroadcastLine(DRAWING_OBJECT, ( CT_RABoard.editingMode==4 ));
		end
	end
end

function CT_RABoard_UpdateBoard(elapsed)
	local board = CT_RABoard;
	if ( DRAWING_OBJECT ) then
		local cx, cy = CT_RABoard_GetCursorPosition();
		if ( LAST_X == cx and LAST_Y == cy ) then
			return;
		end
		LAST_X, LAST_Y = cx, cy;
		CT_RABoard_DrawMovingObject(DRAWING_OBJECT, cx, cy);
		if ( board.editingMode == 2 ) then
			local object = board.objects[DRAWING_OBJECT];
			if ( object ) then
				if ( distA ) then
					-- We have established the line
					local dist = math.abs(distA*cx+distB*cy+distC);
					if ( dist < 4 ) then
						return;
					end
					
					-- Create a new line
					CT_RABoard_EndDrawing("LeftButton");
					CT_RABoard_StartDrawing("LeftButton");
					distA, distB, distC = nil, nil, nil;
				end
				
				if ( not distA ) then
					-- Check if we can establish a new line
					local dx, dy = cx-object.startX, cy-object.startY;
					local l = math.sqrt(dx*dx + dy*dy);
					if ( l > 9 ) then
						distB, distA = dx/l, dy/l;
						distC = -(distA*cx+distB*cy);
					end
				end
			end
		end
	elseif ( board.editingMode == 4 ) then
		local texture = CT_RABoardFrameDrawFrameSelection;
		local cx, cy = CT_RABoard_GetCursorPosition();
		for k, v in board.objects do
			if ( abs(v.startX-cx) < 10 and abs(v.startY-cy) < 10 ) then
				texture:Show();
				texture:SetPoint("TOPLEFT", "CT_RABoardFrameDrawFrame", "BOTTOMLEFT", v.startX-8, v.startY+8);
				texture:SetPoint("BOTTOMRIGHT", "CT_RABoardFrameDrawFrame", "BOTTOMLEFT", v.startX+8, v.startY-8);
				MOUSE_OBJECT = { k, 1 };
				return;
			elseif ( abs(v.endX-cx) < 10 and abs(v.endY-cy) < 10 ) then
				texture:Show();
				texture:SetPoint("TOPLEFT", "CT_RABoardFrameDrawFrame", "BOTTOMLEFT", v.endX-8, v.endY+8);
				texture:SetPoint("BOTTOMRIGHT", "CT_RABoardFrameDrawFrame", "BOTTOMLEFT", v.endX+8, v.endY-8);
				MOUSE_OBJECT = { k };
				return;
			end
		end
		MOUSE_OBJECT = nil;
		texture:Hide();
	end
end

function CT_RABoard_DrawMovingObject(objID, cx, cy)
	-- Note! This code assumes DRAWING_OBJECT is set
	local board = CT_RABoard;
	local obj = board.objects[objID];
	-- Allow for holding of shift to make horizontal/vertical lines
	if ( board.editingMode == 1 and IsShiftKeyDown() ) then
		local xDist = cx-obj.startX;
		local yDist = cy-obj.startY;
		local angle = atan(abs(xDist)/abs(yDist));
		if ( angle >= 45 ) then
			-- Horizontal
			cy = obj.startY;
		else
			-- Vertical
			cx = obj.startX;
		end
	end
	if ( framePositions[1] ) then
		local fw, fh = framePositions[5], framePositions[6];
		if ( cx > fw ) then
			cx = fw;
		elseif ( cx < 0 ) then
			cx = 0;
		end
		if ( cy > fh ) then
			cy = fh;
		elseif ( cy < 0 ) then
			cy = 0;
		end
	end
						-- To be replaced with Iriel's caching code when I get time...
	CT_RABoard_DrawLine(getglobal("CT_RABoardFrameDrawFrameLine" .. obj.objID .. "Texture"), obj.startX, obj.startY, cx, cy, (obj.width+1), obj.r, obj.g, obj.b, obj.a);
end

function CT_RABoard_BroadcastLine(lineID, wasMoved)
	local obj = CT_RABoard.objects[lineID];
	if ( obj ) then
		-- Add type checking here later
		if ( wasMoved ) then
			CT_RA_AddMessage(
				"ML " ..
				floor(obj.startX+0.5) .. " " ..
				floor(obj.startY+0.5) .. " " ..
				floor(obj.endX+0.5) .. " " ..
				floor(obj.endY+0.5) .. " " ..
				lineID
			);
		else
			CT_RA_AddMessage(
				"BL " ..
				floor(obj.startX+0.5) .. " " .. 
				floor(obj.startY+0.5) .. " " .. 
				floor(obj.endX+0.5) .. " " .. 
				floor(obj.endY+0.5) .. " " .. 
				obj.width .. " " .. 
				floor(obj.r*255+0.5) .. " " .. 
				floor(obj.g*255+0.5) .. " " .. 
				floor(obj.b*255+0.5) .. " " .. 
				floor(obj.a*255+0.5) .. " " ..
				lineID
			);
		end
	end
end

function CT_RABoard_InterpretKey(key)
	
end

function CT_RABoard_OnMouseDown(button)
	local board = CT_RABoard;
	local mode = board.editingMode;
	if ( mode == 1 or mode == 2 ) then
		CT_RABoard_StartDrawing(button);
	elseif ( mode == 4 and MOUSE_OBJECT ) then
		CT_RABoardFrameDrawFrameSelection:Hide();
		DRAWING_OBJECT = MOUSE_OBJECT[1];
		if ( MOUSE_OBJECT[2] ) then
			-- Since we are dragging the start, move them around
			local object = board.objects[MOUSE_OBJECT[1]];
			object.startX, object.startY = object.endX, object.endY;
		end
		MOUSE_OBJECT = nil;
	end
end

function CT_RABoard_OnMouseUp(button)
	CT_RABoard_EndDrawing(button);
end

-- Many thanks to Iriel for working out the math & technique behind this code
local lineFactor = (256/254) / 2;

function CT_RABoard_DrawLine(texture, startX, startY, endX, endY, lineWidth, r, g, b, a)
	-- Credits to Iriel for this function
	-- Determine dimensions and center point of line
	local dx,dy = endX - startX, endY - startY;
	local cx,cy = (startX + endX) / 2, (startY + endY) / 2;
	
	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy;
	end
	
	-- Calculate actual length of line
	local l = sqrt((dx * dx) + (dy * dy));
	
	-- Quick escape if it's zero length
	if (l == 0) then
		texture:SetVertexColor(r, g, b, a);
		texture:SetTexCoord(0,0,0,0,0,0,0,0);
		texture:SetPoint("BOTTOMLEFT", CT_RABoardFrameDrawFrame, "BOTTOMLEFT", cx,cy);
		texture:SetPoint("TOPRIGHT",   CT_RABoardFrameDrawFrame, "BOTTOMLEFT", cx,cy);
		return;
	end
	
	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l;
	local sc = s * c;
	
	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((l * c) - (lineWidth * s)) * lineFactor;
		Bhgt = ((lineWidth * c) - (l * s)) * lineFactor;
		BLx, BLy, BRy = (lineWidth / l) * sc, s * s, (l / lineWidth) * sc;
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx; 
		TRy = BRx;
	else
		Bwid = ((l * c) + (lineWidth * s)) * lineFactor;
		Bhgt = ((lineWidth * c) + (l * s)) * lineFactor;
		BLx, BLy, BRx = s * s, -(l / lineWidth) * sc, 1 + (lineWidth / l) * sc;
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
		TRx = TLy;
	end
	
	-- Set texture coordinates and anchors
	local xBL, yBL, xTR, yTR;
	texture:SetVertexColor(r, g, b, a);
	texture:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy);
	texture:SetPoint("BOTTOMLEFT", CT_RABoardFrameDrawFrame, "BOTTOMLEFT", cx - Bwid, cy - Bhgt);
	texture:SetPoint("TOPRIGHT",   CT_RABoardFrameDrawFrame, "BOTTOMLEFT", cx + Bwid, cy + Bhgt);
	return 1;
end

------------------------------------------------------------------------
--                            Options                                 --
------------------------------------------------------------------------
function CT_RABoard_ChangeMode(mode)
	CT_RABoard.editingMode = mode;
	for i = 1, 4, 1 do
		if ( mode == i ) then
			getglobal("CT_RABoardFrameOptions" .. i):SetChecked(1);
		else
			getglobal("CT_RABoardFrameOptions" .. i):SetChecked(nil);
		end
	end
	-- Update the slider
	CT_RABoard_Options_UpdateSlider();
	
	-- Reset the LAST_ANGLE variable
	LAST_ANGLE = nil;
end

function CT_RABoard_Load()
	-- Load the line tool
	CT_RABoard_ChangeMode(1);
	
	-- Initialize slash commands and escape-closing of the frame
	tinsert(UISpecialFrames, "CT_RABoardFrame");
	SLASH_RAWB1 = "/rawhiteboard";
	SLASH_RAWB2 = "/rawb";
	SlashCmdList["RAWB"] = function()
		if ( not CT_RABoardFrame:IsVisible() ) then
			ShowUIPanel(CT_RABoardFrame);
		else
			HideUIPanel(CT_RABoardFrame);
		end
	end
	
	-- Set the color of the color swatch button
	local color = CT_RABoard.color;
	CT_RABoardFrameOptionsColorNormalTexture:SetVertexColor(color.r, color.g, color.b);
	
	-- Set the slider values
	CT_RABoardFrameOptionsSlider:SetMinMaxValues(1, 5);
	CT_RABoardFrameOptionsSlider:SetValueStep(1);
end

-- Color Swatch
function CT_RABoard_Options_ShowColorSwatch()
	local frame = this:GetParent();
	local color = CT_RABoard.color;
	frame.r = color.r;
	frame.g = color.g;
	frame.b = color.b;
	frame.opacity = color.a;
	frame.opacityFunc = CT_RABoard_Options_SetColorSwatchOpacity;
	frame.swatchFunc = CT_RABoard_Options_SetColorSwatchColor;
	frame.cancelFunc = CT_RABoard_Options_CancelColorSwatch;
	frame.hasOpacity = 1;
	CloseMenus();
	UIDropDownMenuButton_OpenColorPicker(frame);
end

function CT_RABoard_Options_SetColorSwatchColor()
	local r, g, b = ColorPickerFrame:GetColorRGB();
	local color = CT_RABoard.color;
	color.r = r;
	color.g = g;
	color.b = b;
	CT_RABoardFrameOptionsColorNormalTexture:SetVertexColor(r, g, b);
end

function CT_RABoard_Options_SetColorSwatchOpacity()
	CT_RABoard.color.a = OpacitySliderFrame:GetValue();
end

function CT_RABoard_Options_CancelColorSwatch(val)
	local color = CT_RABoard.color;
	color.r = val.r;
	color.g = val.g;
	color.b = val.b;
	color.a = val.opacity;
	CT_RABoardFrameOptionsColorNormalTexture:SetVertexColor(val.r, val.g, val.b);
end

-- Slider
function CT_RABoard_Options_UpdateSlider()
	local mode = CT_RABoard.editingMode;
	local value;
	if ( mode == 4 ) then -- Edit tool
		CT_RABoardFrameOptionsSlider:Hide();
		return;
	elseif ( mode == 1 ) then -- Line tool
		value = ( CT_RABoard.lineWidth or 3 );
		CT_RABoardFrameOptionsSliderText:SetText("Line Width - " .. value);
		CT_RABoardFrameOptionsSliderHigh:SetText("Wide");
		CT_RABoardFrameOptionsSliderLow:SetText("Thin");
	elseif ( mode == 2 ) then -- Text tool
		value = ( CT_RABoard.textSize or 3 );
		CT_RABoardFrameOptionsSliderText:SetText("Text Size - " .. value);
		CT_RABoardFrameOptionsSliderHigh:SetText("Big");
		CT_RABoardFrameOptionsSliderLow:SetText("Small");
	elseif ( mode == 3 ) then -- Object tool
		value = ( CT_RABoard.objectSize or 3 );
		CT_RABoardFrameOptionsSliderText:SetText("Object Size - " .. value);
		CT_RABoardFrameOptionsSliderHigh:SetText("Big");
		CT_RABoardFrameOptionsSliderLow:SetText("Small");
	end
	CT_RABoardFrameOptionsSlider:Show();
	CT_RABoardFrameOptionsSlider:SetValue(value);
end

function CT_RABoard_Options_UpdateSliderValue()
	local mode = CT_RABoard.editingMode;
	if ( mode == 1 ) then
		CT_RABoard.lineWidth = this:GetValue();
	elseif ( mode == 2 ) then
		CT_RABoard.textSize = this:GetValue();
	elseif ( mode == 3 ) then
		CT_RABoard.objectSize = this:GetValue();
	else
		return;
	end
	CT_RABoard_Options_UpdateSlider();
end