--PetExtend.Utils.lua
--Holds all functions that provide internal utility but no external value.


--Basic Util Functions
function PetExtend_ConsoleMessage(msg, form)
	--Purpose: Provides a handy way of sending console messages pre-stamped.
	
	--Colors
	local blue = "|cCF3366FE";
	local yellow = "|cFFFFFFCC";
	local green = "|cFF66FF66";
	local red = "|cFFFF6666";
	local colorEnd = "|r"
	local output = "";
	
	if (msg ~= nil) then
		if (form == nil) then
			output = (yellow..msg..colorEnd);
		elseif (form == "help") then
			output = (blue..msg..colorEnd);
		elseif (form == "stamped") then
			output = (blue.."[PetExtend]"..colorEnd..yellow..": "..msg..colorEnd);
		elseif (form == "error") then
			output = (blue.."[PetExtend]"..colorEnd..red..": Error"..colorEnd..yellow.." - "..msg..colorEnd);
		else
			output = (yellow..msg..colorEnd);
		end
		
		DEFAULT_CHAT_FRAME:AddMessage(output);
	end
end


--Register Functions
function PetExtend_RegisterCalloff(reason, timeperiod)
	--Purpose: Handy way to handle calloffs. Plays sound, displays message, and puts calloff in the callOff queue.
	--Notes: Perhaps change the way that you register. Maybe a table or somesuch.

	local message = "";
	
	if (reason == "nochase") then
		message = UnitName("pettarget").." is running! Pet disengage?";
	elseif (reason == "healthwarning") then
		message = "Pet health low! Pet disengage?";
	else
		message = reason.." calloff registered. Pet disengage?"
	end
	
	if (PetExtend_Config.calloffaudio == true) then
		PlaySound("Fishing Reel in");
	end
	
	UIErrorsFrame:AddMessage(message, 1.0, 1.0, 0.0, 1.0, 2.5);
	PetExtend_smartButton_callOff = GetTime() + timeperiod;
end

function PetExtend_AddFocusPriority(skill, priority)
	--Purpose: Adding a skill to the focus priority table at a specified position or if there is not one, at the last position.

	--Check if pet exists and if the player is a hunter
	if (((not UnitExists("pet")) or (UnitIsDead("pet"))) or (UnitClass("player") ~= PE_HUNTER)) then
		return;
	end
	
	if (getn(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"]) ~= 0) then
		--previous priorities in table
		for i=1, getn(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"]), 1 do
			--Go through all the previous ones to search for the requested one.
			if ((PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"][i] == skill)) then
				if ((priority ~= nil) and (i ~= priority)) then
					--not in right order; remove and readd at set priority
					tremove(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], i);
					tinsert(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], priority, skill);
					return;
				else
					--not in right order; remove and readd at last priority
					tremove(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], i);
					tinsert(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], skill);
					return;
				end
			end
		end
	end
	
	--If code gets this far, it was not in the previous priorities
	if (priority ~= nil) then
		--priority specified; add at specified position
		tinsert(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], priority, skill);
		--what happens when the priority isn't one and there are none previous?
	else
		--priority not specified; add at last postion
		tinsert(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"], skill);
	end
	
end


--Calloff Functions
function PetExtend_ScatterShotCalloff()
	--Purpose: Sets pet to wait or follow based on set reaction. This fires on "successful" scattershot.

	if ((PetExtend_Config.enabled == false) or ((not UnitExists("pet")) or (UnitIsDead("pet")))) then
		return;
	end
	
	if (PetExtend_Config.scattershot_enabled == true) then
		if (PetExtend_Config.calloffaudio == true) then
			PlaySound("Fishing Reel in");
		end
		if (PetExtend_Config.scattershot_reaction == "Stay") then
			PetWait();
		elseif (PetExtend_Config.scattershot_reaction == "Follow") then
			PetFollow();
		elseif (PetExtend_Config.scattershot_reaction == "Passive") then
			PetExtend_SwitchMood("PET_MODE_PASSIVE");
		end
	end
	
end

function PetExtend_PvPFeignCalloff()
	--Purpose: Sets pet to wait or follow based on set reaction. This fires on "successful" feign death while in PvP.

	if ((PetExtend_Config.enabled == false) or ((not UnitExists("pet")) or (UnitIsDead("pet")))) then
		return;
	end
	
	if ((PetExtend_Config.pvpfeign_enabled == true) and (PetExtend_InPVP == true)) then
		if (PetExtend_Config.calloffaudio == true) then
			PlaySound("Fishing Reel in");
		end
		if (PetExtend_Config.pvpfeign_reaction == "Stay") then
			PetWait();
		elseif (PetExtend_Config.pvpfeign_reaction == "Follow") then
			PetFollow();
		elseif (PetExtend_Config.pvpfeign_reaction == "Passive") then
			PetExtend_SwitchMood("PET_MODE_PASSIVE");
		end
	end
	
end


--Pet Util Functions
function PetExtend_ScanPetBar()
	--Purpose: The almighty PetBar scan! This loops through all of the possible PetBar slots and gets the id number for each petskill
	
	if (UnitExists("pet")) then
		--Reset Vars
		if (UnitClass("player") == PE_HUNTER) then
			PetExtend_PetAction[PE_BITE].id = -1;
			PetExtend_PetAction[PE_CLAW].id = -1;
			PetExtend_PetAction[PE_GROWL].id = -1;
			PetExtend_PetAction[PE_DASH].id = -1;
			PetExtend_PetAction[PE_DIVE].id = -1;
			PetExtend_PetAction[PE_PROWL].id = -1;
			PetExtend_PetAction[PE_CHARGE].id = -1;
			PetExtend_PetAction[PE_STOMP].id = -1;
			PetExtend_PetAction[PE_LIGHTNING].id = -1;
			PetExtend_PetAction[PE_HOWL].id = -1;
			PetExtend_PetAction[PE_SCREECH].id = -1;
			PetExtend_PetAction[PE_SHELL].id = -1;
			PetExtend_PetAction[PE_COWER].id = -1
			PetExtend_PetAction[PE_POISON].id = -1;	
		elseif (UnitClass("player") == PE_WARLOCK) then
			PetExtend_PetAction[PE_DEVOUR].id = -1;
			PetExtend_PetAction[PE_PARANOIA].id = -1;
			PetExtend_PetAction[PE_SPELLLOCK].id = -1;
			PetExtend_PetAction[PE_TAINTEDBLOOD].id = -1;
			PetExtend_PetAction[PE_BLOODPACT].id = -1;
			PetExtend_PetAction[PE_FIRESHIELD].id = -1;
			PetExtend_PetAction[PE_FIREBOLT].id = -1;
			PetExtend_PetAction[PE_PHASESHIFT].id = -1;
			PetExtend_PetAction[PE_LASH].id = -1;
			PetExtend_PetAction[PE_INVIS].id = -1;
			PetExtend_PetAction[PE_SEDUCTION].id = -1;
			PetExtend_PetAction[PE_KISS].id = -1;
			PetExtend_PetAction[PE_CONSUME].id = -1;
			PetExtend_PetAction[PE_SACRIFICE].id = -1;
			PetExtend_PetAction[PE_SUFFERING].id = -1;
			PetExtend_PetAction[PE_TORMENT].id = -1;
		end
		
		--Go through all pet action slots and set the id's 
		for i=1, NUM_PET_ACTION_SLOTS, 1 do
			local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(i);
			if (name ~= nil) then
				if (PetExtend_PetAction[name]) then
					PetExtend_PetAction[name].id = i;
					if (subtext ~= nil) then
						if (strfind(subtext, "%d")) then
							local subtextNumber = strfind(subtext, "%d");
							PetExtend_PetAction[name].rank = tonumber(strsub(subtext, subtextNumber, subtextNumber));
							if ((PetExtend_PetAction[name].costDB) and (PetExtend_PetAction[name].costDB[PetExtend_PetAction[name].rank])) then
								PetExtend_PetAction[name].cost = PetExtend_PetAction[name].costDB[PetExtend_PetAction[name].rank]
							end
						end
					end
				end
			end
		end
	end
	
	
end

function PetExtend_PetHasSkill(skill)
	--Purpose: Provides an easy to call check for whether a pet has a skill or not.

	--Check if pet exists
	if ((not UnitExists("pet")) or (UnitIsDead("pet"))) then
		return;
	end
	
	--Return true/false based on if the pet has skill
	if (PetExtend_PetAction[skill].id > 0) then
		return true;
	else
		return false;
	end
end

function PetExtend_EnableAutoCast(skill)
	--Purpose: Enable autocast on a specified skill. Also very useful.
	
	--Check if pet exists
	if ((not UnitExists("pet")) or (UnitIsDead("pet"))) then
		return;
	end

	if (PetExtend_PetHasSkill(skill)) then
		--Scan the pet ability
		local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(PetExtend_PetAction[skill].id);
		--Set auto cast to enable
		if (not autoCastEnabled) then
			TogglePetAutocast(PetExtend_PetAction[skill].id);
		end
	end
end

function PetExtend_DisableAutoCast(skill)
	--Purpose: Disables autocasts on a specified skill. Very useful.
	--Check if pet exists
	if ((not UnitExists("pet")) or (UnitIsDead("pet"))) then
		return;
	end

	if (PetExtend_PetHasSkill(skill)) then
		--scan the pet ability
		local name, subtext, texture, isToken, isActive, autoCastAllowed, autoCastEnabled = GetPetActionInfo(PetExtend_PetAction[skill].id);
		--Set autocast to disable
		if (autoCastEnabled) then
			TogglePetAutocast(PetExtend_PetAction[skill].id);
		end
	end
end

function PetExtend_CheckCooldownLeft(skill)
	--Purpose: Provide an easy to call cooldown checker for pet skills for myself.

	--Check if pet exists
	if ((not UnitExists("pet")) or (UnitIsDead("pet"))) then
		return;
	end
	
	--Return the cooldown left, or 0
	if (PetExtend_PetHasSkill(skill)) then
		local startTime, duration, enable = GetPetActionCooldown(PetExtend_PetAction[skill].id);
		if not (startTime == 0) then
			local cooldownLeft = duration - (GetTime() - startTime);
			return cooldownLeft;
		else
			return 0;
		end
	end
end

function PetExtend_SwitchMood(mood)
	if (mood == "PET_MODE_PASSIVE") then
		PetExtend_MoodSwitch = true;
		PetPassiveMode();
		PetExtend_MoodSwitch = false;
	elseif (mood == "PET_MODE_AGGRESSIVE") then
		PetExtend_MoodSwitch = true;
		PetAggressiveMode();
		PetExtend_MoodSwitch = false;
	elseif (mood == "PET_MODE_DEFENSIVE") then
		PetExtend_MoodSwitch = true;
		PetDefensiveMode();
		PetExtend_MoodSwitch = false;
	elseif (mood == "default") then
		PetExtend_SwitchMood(PetExtend_Config.selectedMood)
	end
end

function PetExtend_ChangeMode(mode)
	
	if ((UnitName("pet") ~= nil) and (UnitName("pet") ~= PE_UNKNOWN_ENTITY) and (UnitName("pet") ~= PE_UNKNOWN)) then
		if (PetExtend_Config[UnitName("pet")][mode]) then
			PetExtend_Config[UnitName("pet")].currentmode = mode;
			PetExtend_ModeScrollBar_Update();
			PetExtend_Update_FocusManagement();
			PetExtend_Update_SkillManagement();
			PetExtend_PetActionBar_Update();
			PetExtend_UpdateAutoCasts();
			PetExtend_UpdateStatusFrame();
			if (PetExtend_GUIFrame:IsVisible()) then
				PetExtend_GUIFrame:Hide();
				PetExtend_GUIFrame:Show();
			end
			return true;
		else
			return nil;
		end
	end
	
end

function PetExtend_FocusRegen(skill)
	--Purpose: Return how much focus will be regained during a skills cooldown.

	--Check if pet exists and if the player is hunter
	if (((not UnitExists("pet")) or (UnitIsDead("pet"))) or (UnitClass("player") ~= PE_HUNTER)) then
		return;
	end
	
	--When skill will be able to cast minus the next focus time
	if ((PetExtend_CheckCooldownLeft(skill) ~= nil) and (PetExtend_NextFocusTime ~= nil)) then
		local modCoolFocus = ceil(((PetExtend_CheckCooldownLeft(skill) + GetTime()) - PetExtend_NextFocusTime)/5);
		if (PetExtend_CheckCooldownLeft(skill) == 0) then
			return 0;
		elseif ( (PetExtend_CheckCooldownLeft(skill) + GetTime() ) < (PetExtend_NextFocusTime) ) then
			return 0;
		else
			return (PETEXTEND_FOCUSPERTICK * modCoolFocus);
		end
	end
end


--Utilities that check stuff
function PetExtend_CheckPVP()
	--Purpose: Set PVP var based on PVP status of hostile target.
	
	if ((UnitCanAttack("player", "target")) and (UnitPlayerControlled("target"))) then
		PetExtend_InPVP = true;
		return true;
	else
		PetExtend_InPVP = false;
		return false;
	end
end

function PetExtend_MarkOn()
	--Purpose: Provides a check that scans UnitDebuffs for the presence of the "Sniper" texture.
	--Todo: Change to better way of scanning buffs, maybe check the rank as well.

	local pe_counter = 1;
	while (UnitDebuff("target", pe_counter)) do
		if (string.find(UnitDebuff("target", pe_counter), "Sniper")) then
			return true;
		end
		pe_counter = pe_counter + 1;
	end
	return false;
end

function PetExtend_BreakableDebuffCheck()
	--Purpose: Checks debuffs for breakable debuffs. Called by smart button.
	
	for i = 1, 16 do
		if (UnitDebuff("target", i)) then
			PetExtend_Tooltip:ClearLines()
			PetExtend_Tooltip:SetUnitDebuff("target", i);
			local debuffName = PetExtend_TooltipTextLeft1:GetText();
				for i = 1, getn(PETEXTEND_BREAKABLEDEBUFFS), 1 do
					if (debuffName == PETEXTEND_BREAKABLEDEBUFFS[i]) then
						return true;
					end
				end
		elseif (not UnitDebuff("target", i)) then
			return false;
		end
	end
end

function PetExtend_CheckTalents()
	if (UnitClass("player") == PE_HUNTER) then
		--Check talents for focus one and compensate
		local nameTalent , iconPath , iconX , iconY , currentRank , maxRank = GetTalentInfo(1, 14);
		if (currentRank == 0) then
			PETEXTEND_FOCUSPERTICK = 24;
		elseif (currentRank == 1) then
			PETEXTEND_FOCUSPERTICK = (floor(24 * 1.10));
		elseif (currentRank == 2) then
			PETEXTEND_FOCUSPERTICK = (floor(24 * 1.20));
		end
	end
end

function PetExtend_IsSkillInFocusPriority(skill)

	--Check if pet exists
	if ((UnitName("pet") ~= nil) and (UnitName("pet") ~= PE_UNKNOWN_ENTITY) and (UnitName("pet") ~= PE_UNKNOWN)) then
		for i=1, getn(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"]) do
			if (PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Focus Priority"][i] == skill) then
				return i;
			end
		end
		
		--if it got this far then it is not in the focus priority
		return nil;
	end
end

function PetExtend_IsSkillInSkillManagement(skill)

	--Check if pet exists
	if ((UnitName("pet") ~= nil) and (UnitName("pet") ~= PE_UNKNOWN_ENTITY) and (UnitName("pet") ~= PE_UNKNOWN)) then
		for i=1, getn(PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Skill Management"]) do
			if (PetExtend_Config[UnitName("pet")][PetExtend_Config[UnitName("pet")].currentmode]["Skill Management"][i] == skill) then
				return i;
			end
		end
		
		--if it got this far then it is not in the focus priority
		return nil;
	end
	
end
