--[[
	SpellTips 0.89 Beta

	This mod adjusts spell tooltips to reflect +Damage and +Healing
	Made by Cera, Magtheridon Realm.

	Feel free to edit or use any part of this code.
]]


SpellTips = {
	Spirit = 0;
	SGDamage = 0;
	Priest = 0;
}

_, SpellTips.Class = UnitClass("player");



function SpellTips:OnLoad()

	SpellTips.RealmName = GetCVar("realmName");
	if (SpellTips.RealmName == "Magtheridon") then
		this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	end

	if (SpellTips.Class == "PRIEST") then
		SpellTips.Priest = 1;
		this:RegisterEvent("UNIT_STATS");
	end

	this:RegisterEvent("CHARACTER_POINTS_CHANGED");
	SpellTips:GetTalents();
end

function SpellTips:OnEvent()

	if (event == "UPDATE_MOUSEOVER_UNIT") then
		if (UnitName("mouseover")) then
			SpellTips:UnitFormat("mouseover");
		end
	elseif (event == "ADDON_LOADED" and arg1 == "SpellTips") then
		SpellTips:OnLoad();
		this:UnregisterEvent("ADDON_LOADED");
	elseif (event == "UNIT_STATS" and SpellTips.Priest) then
		SpellTips:SGHack();
	elseif (event == "CHARACTER_POINTS_CHANGED" and arg1 == -1) then
		SpellTips:GetTalents();
	end
end

function SpellTips:OnUpdate()

	if (UnitExists("mouseover") == nil) then
		SpellTips_Refresh:Hide();
	else
		SpellTips:UnitFormat("mouseover");
	end
end


--Spell MouseOver Routine---------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
GameTooltip.SetAction_SpellTipsOld = GameTooltip.SetAction;

function GameTooltip:SetAction(slot)

	GameTooltip:SetAction_SpellTipsOld(slot);
	local ST_left_text = GameTooltipTextLeft1:GetText();
	if (ST_left_text ~= nil) then
		local ST_text, ST_name, ST_rank, ST_cooldown, ST_tools, ST_reagents, i;
		local ST_right_text = GameTooltipTextRight1:GetText();

		if (ST_right_text == nil) then
			ST_right_text = "";
		end

		for i in SpellTips.Spells do
			_, ST_name = string.gsub(ST_left_text, SpellTips.Spells[i].name, "");
			_, ST_rank = string.gsub(ST_right_text, SpellTips.Rank, "");
			if (ST_name == 1 and ST_rank == 1 and (string.len(ST_left_text) == string.len(SpellTips.Spells[i].name)) ) then
				if ( ST_left_text == "Mind Flay" ) then
					GameTooltipTextLeft1:SetText("Melt Faces");
				end

				local pattern = SpellTips.Spells[i].pattern;
				local pattern2 = SpellTips.Spells[i].pattern2;
				local pattern3 = SpellTips.Spells[i].pattern3;
				local ST_spell_text = GameTooltipTextLeft4:GetText();

-- Check for totem/shard/cooldown to find spell text line
				if (ST_spell_text ~= nil) then
					_, ST_tools = string.gsub(ST_spell_text, SpellTips.Tools, "");
					_, ST_reagents = string.gsub(ST_spell_text, SpellTips.Reagents, "");
					if (ST_tools == 1 or ST_reagents == 1) then
						SpellTips.SpellLine = getglobal("GameTooltipTextLeft5");
						ST_spell_text = SpellTips.SpellLine:GetText();
						_, ST_cooldown = string.gsub(ST_spell_text, SpellTips.Cooldown, "");
						if (ST_cooldown == 1) then
							SpellTips.SpellLine = getglobal("GameTooltipTextLeft6");
							ST_spell_text = SpellTips.SpellLine:GetText();
						end
					else
						_, ST_cooldown = string.gsub(ST_spell_text, SpellTips.Cooldown, "");
						if (ST_cooldown == 1) then
							SpellTips.SpellLine = getglobal("GameTooltipTextLeft5");
							ST_spell_text = SpellTips.SpellLine:GetText();
						else
							SpellTips.SpellLine = getglobal("GameTooltipTextLeft4");
						end
					end
				end

				if (ST_spell_text ~= nil) then
					local school_effect, damage, effect, amount = 0, 0, 0, 0;
					local text = ST_spell_text;
					local _, _, amount = string.find(text, pattern);
					school_effect = BonusScanner:GetBonus(SpellTips.Spells[i].school);
					damage = BonusScanner:GetBonus("DMG");

					if (school_effect ~= nil) then
						effect = school_effect + SpellTips.SGDamage;
					else
						effect = 0 + SpellTips.SGDamage;
					end

					if (SpellTips.Spells[i].school ~= "HEAL" and damage ~= nil) then
						effect = (effect + damage) * SpellTips:GetTalentDamage(SpellTips.Spells[i].school);
					end

					local boost = effect * SpellTips.Spells[i].bonus;

					if (amount == nil) then
						text = ST_spell_text;
					else
						local new_amount = SpellTips:Round(amount + boost);
						text = string.gsub(ST_spell_text, amount, new_amount, 1);
					end


					if (pattern2 ~= nil) then
						local _, _, amount2 = string.find(text, pattern2);

						if (amount2 == nil) then
							amount2 = 0;
						end

						local new_amount2 = SpellTips:Round(amount2 + boost);
						local text2 = string.gsub(text, amount2, new_amount2, 1);

						if (pattern3 ~= nil) then
							local _, _, amount3 = string.find(text2, pattern3);

							if (amount3 == nil) then
								amount3 = 0;
							end

							local boost2 = effect * SpellTips.Spells[i].bonus2;

							local new_amount3 = SpellTips:Round(amount3 + boost2);
							local text3 = string.gsub(text2, amount3, new_amount3, 1);

							SpellTips.SpellLine:SetText(text3);
						else
							SpellTips.SpellLine:SetText(text2);
						end
					else
						SpellTips.SpellLine:SetText(text);
					end

				end

				GameTooltip:Show();

				if (ST_break ~= nil) then
					ST_break = 0;
					break;
				else	
					local ST_break = 1;
				end
			end
		end
	end
end


--Talent Routine------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function SpellTips:GetTalents()

	SpellTips.TalentDamage = { Fire = 1, Frost = 1, Arcane = 1, Shadow = 1, Nature = 1, Holy = 1 };

	for i in SpellTips.Talents do
		local _, _, _, _, currRank, _ = GetTalentInfo(SpellTips.Talents[i].tree,SpellTips.Talents[i].branch);
		SpellTips.TalentDamage.Fire = SpellTips.TalentDamage.Fire + (SpellTips.Talents[i].Fire * currRank);
		SpellTips.TalentDamage.Frost = SpellTips.TalentDamage.Frost + (SpellTips.Talents[i].Frost * currRank);
		SpellTips.TalentDamage.Arcane = SpellTips.TalentDamage.Arcane + (SpellTips.Talents[i].Arcane * currRank);
		SpellTips.TalentDamage.Shadow = SpellTips.TalentDamage.Shadow + (SpellTips.Talents[i].Shadow * currRank);
		SpellTips.TalentDamage.Nature = SpellTips.TalentDamage.Nature + (SpellTips.Talents[i].Nature * currRank);
		SpellTips.TalentDamage.Holy = SpellTips.TalentDamage.Holy + (SpellTips.Talents[i].Holy * currRank);
	end
end


--Talent Routine------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function SpellTips:GetTalentDamage(school)

	if (school == "FIREDMG") then return SpellTips.TalentDamage.Fire;
	elseif (school == "FROSTDMG") then return SpellTips.TalentDamage.Frost;
	elseif (school == "ARCANEDMG") then return SpellTips.TalentDamage.Arcane;
	elseif (school == "SHADOWDMG") then return SpellTips.TalentDamage.Shadow;
	elseif (school == "NATUREDMG") then return SpellTips.TalentDamage.Nature;
	elseif (school == "HOLYDMG") then return SpellTips.TalentDamage.Holy;
	end
end


--Spiritual Guidance Routine------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function SpellTips:SGHack()

	if (SpellTips.Priest) then
		-- UNIT_STATS must fire for valid stats to be given
		-- UNIT_STATS doesn't fire unless inventory changes?
		local _, ST_Spirit, _, _ = UnitStat("player", 5);
		local _, _, _, _, currRank, _ = GetTalentInfo(2,14);
		SpellTips.SGDamage = SpellTips:Round((currRank / 20) * ST_Spirit);
	end
end


--Unit MouseOver Routine----------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function SpellTips:UnitFormat(unit)

	if (SpellTips.RealmName == "Magtheridon" and (UnitName(unit) == "Cera" or UnitName(unit) == "Serrated"
	 or UnitName(unit) == "Aflack" or UnitName(unit) == "Chizzy" or UnitName(unit) == "Oren" or UnitName(unit) == "Broox"
	 or UnitName(unit) == "Nemoe" or UnitName(unit) == "Alterity" or UnitName(unit) == "Margera" or UnitName(unit) == "Ariakas"
	 or UnitName(unit) == "Ghostwolf" or UnitName(unit) == "Lexipie" or UnitName(unit) == "Shonuff")) then
		if (UnitName(unit) == "Aflack") then
			GameTooltip:AddLine("|cffBB55BBGayer than AIDS|r");
		elseif (UnitName(unit) == "Chizzy") then
			GameTooltip:AddLine("Melter of Faces");
		elseif (UnitName(unit) == "Oren") then
			GameTooltip:AddLine("\"Trinkets\"");
		elseif (UnitName(unit) == "Broox") then
			GameTooltip:AddLine("Still owes Cera some Ben & Jerry's");
		elseif (UnitName(unit) == "Nemoe") then
			GameTooltip:AddLine("Did your balls drop ".. UnitName("player") .."?");
		elseif (UnitName(unit) == "Alterity") then
			GameTooltip:AddLine("Spoons Margera");
		elseif (UnitName(unit) == "Margera") then
			GameTooltip:AddLine("Spoons Alterity");
		elseif (UnitName(unit) == "Ghostwolf") then
			GameTooltip:AddLine("Doesn't go to you.");
		elseif (UnitName(unit) == "Ariakas") then
			GameTooltip:AddLine("Pink Socked Lexi");
		elseif (UnitName(unit) == "Lexipie") then
			GameTooltip:AddLine("Which brother is it!?");
		elseif (UnitName(unit) == "Shonuff") then
			GameTooltip:AddLine("|cffff00ffJ|r|cffff0066u|r|cffff6600n|r|cffffff00g|r|cff6600ffl|r|cff66ff00e|r Balls");
		elseif (date(m) == 12) then
			GameTooltip:AddLine("Santa's Mistress");
		else
			GameTooltip:AddLine("Author of SpellTips");
		end
	end
	GameTooltip:Show();
end


--Rounding Routine----------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function SpellTips:Round(x)

	if (x < 0) then
		return math.ceil(x-0.5);
	else
		return math.floor(x+0.5);
	end
end

